/*
 *  AFS - The native AtheOS file-system
 *  Copyright (C) 1999 - 2001 Kurt Skauen
 *  Copyright (C) 2001 - Mike Bennett (mike@stressbunny.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  CHANGES
 *  =======
 *   - 15/Oct/2001 (Mike Bennett): Took from AtheOS 0.3.6 to use in
 *     debugafs
 */

#include<unistd.h>

#include "afs.h"
#include "btree.h"
#include "posix/errno.h"

#include<stdio.h>
#include<stdlib.h>

int afs_read_superblock(int nDev, AfsVolume_s *psVolume, AfsSuperBlock_s *psSuperBlock)
{
    int64		nUsedBlocks;
    int			nBlockSize;
    int			nError = 0;

//    memset(psVolume, 0, sizeof(AfsVolume_s));

//    nError = afs_init_hash_table( &psVolume->av_sLogHashTable );
 
    if(nError < 0) {
        goto error2;
    }

    psVolume->av_psSuperBlock = psSuperBlock;

    printf("Load super block\n");
 
//    psVolume->av_nSuperBlockPos = 1024;

    lseek(nDev, psVolume->av_nSuperBlockPos, SEEK_SET);

    if(read(nDev, psSuperBlock, AFS_SUPERBLOCK_SIZE) != AFS_SUPERBLOCK_SIZE) {
        nError = -EIO;
        printf("Failed to read super block\n");
        goto error4;
    }

	return (0);
 
    if(afs_validate_superblock(psSuperBlock, false) == false) {
        psVolume->av_nSuperBlockPos = 0;

	lseek(nDev, psVolume->av_nSuperBlockPos, SEEK_SET);

        if(read(nDev, psSuperBlock, AFS_SUPERBLOCK_SIZE ) != AFS_SUPERBLOCK_SIZE) {
            printf("Failed to read super block\n");
            nError = -EIO;
            goto error4;
        }
    }

	printf("\n");
 
    printf("Validate super block\n");

    if(afs_validate_superblock(psSuperBlock, false) == false) {
        printf( "Invalide super block\n" );
        nError = -EINVAL;
        goto error4;
    }
 
    nBlockSize = psSuperBlock->as_nBlockSize;
 
    if(psVolume->av_nSuperBlockPos == 0) {
        printf("Warning: afs_mount() old filesystem format\n");
        psVolume->av_nBitmapStart = 1;
    } else {
        psVolume->av_nBitmapStart = (psSuperBlock->as_nBootLoaderSize * nBlockSize + AFS_SUPERBLOCK_SIZE + 1024 + nBlockSize - 1) / nBlockSize;
    }
 
    psVolume->av_nJournalSize = psSuperBlock->as_nLogSize;
    psVolume->av_nLoggStart   = psSuperBlock->as_nLogStart;
    psVolume->av_nLoggEnd     = psVolume->av_nLoggStart + psVolume->av_nJournalSize - 1;
 
    printf("Log start: %Ld BitmapStart : %Ld\n", psVolume->av_nLoggStart, psVolume->av_nBitmapStart);
 
//    printk( "Bitmap : %Ld -> %Ld  Log : %Ld -> %Ld\n", psVolume->av_nBitmapStart, psVolume->av_nBitmapStart + psSuperBlock->as_nAllocGroupCo 
//        psVolume->av_nFsID   = nFsID;
//            psVolume->av_nDevice = nDev; 


    psVolume->av_nFsID   = 0;		// Mike: nFsID;
    psVolume->av_nDevice = nDev;
/* 
    nError = setup_device_cache( nDev, nFsID, psSuperBlock->as_nNumBlocks );
 
    if ( nError < 0 ) {
        printk( "Error: afs_mount() failed to setup device cache\n" );
        goto error4;
    }
 
    psVolume->av_hJournalLock = create_semaphore( "afs_journal_mutex", 1, SEM_REQURSIVE );
 
    if ( psVolume->av_hJournalLock < 0 ) {
        printk( "Error : afs_mount() failed to create journal semaphore\n" );
        nError = psVolume->av_hJournalLock;
        goto error4;
    }
    psVolume->av_hJournalListsLock = create_semaphore( "afs_journal_lists_mutex", 1, SEM_REQURSIVE );
 
    if ( psVolume->av_hJournalLock < 0 ) {
        printk( "Error : afs_mount() failed to create journal lists semaphore\n" );
        nError = psVolume->av_hJournalLock;
        goto error5;
    }
 
 
    psVolume->av_hBlockListMutex = create_semaphore( "afs_buffer_lock", 1, SEM_REQURSIVE );
    if ( psVolume->av_hBlockListMutex < 0 ) {
        printk( "Error : afs_mount() failed to create block list semaphore\n" );
        nError = psVolume->av_hBlockListMutex;
        goto error6;
    }
 
    psVolume->av_hIndexDirMutex = create_semaphore( "afs_buffer_lock", 1, SEM_REQURSIVE );
    if ( psVolume->av_hIndexDirMutex < 0 ) {
        printk( "Error : afs_mount() failed to create file-index semaphore\n" );
        nError = psVolume->av_hIndexDirMutex;
        goto error7;
    }

    nError = afs_replay_log( psVolume );
    if ( nError < 0 ) {
        printk( "Error: afs_mount() failed to replay journal\n" );
        goto error8;
    }

    *ppVolData = psVolume;
    *pnRootIno = afs_run_to_num( psSuperBlock, &psSuperBlock->as_sRootDir );
 
    nUsedBlocks = afs_count_used_blocks( psVolume );
 
    if ( psSuperBlock->as_nUsedBlocks != nUsedBlocks ) {
        printk( "Error: afs_mount() Inconsistency between super-block and allocation bitmap!\n" );
        printk( "Superblock: %Ld used blocks\n", psSuperBlock->as_nUsedBlocks );
        printk( "Bitmap:     %Ld used blocks\n", nUsedBlocks );
        psSuperBlock->as_nUsedBlocks = nUsedBlocks;
    }
 
    printk( "afs_mount() : filesystem has %Ld free blocks (of %Ld)\n",
            (psSuperBlock->as_nNumBlocks - psSuperBlock->as_nUsedBlocks), psSuperBlock->as_nNumBlocks );
 
 
    nError = afs_empty_delme_dir( psVolume );
    if ( nError < 0 ) {
        printk( "Error: failed to cleanup delete-me directory!\n" );
        goto error8;
    }
    psVolume->av_bRunJournalFlusher = true;
    nError = afs_start_journal_flusher( psVolume );
    if ( nError < 0 ) {
        printk( "afs_mount() failed to start journal flushing thread\n" );
        goto error8;
    }
*/
    printf( "AFS filesystem superblock successfully read\n" );

    return( 0 );

error8:
//    delete_semaphore( psVolume->av_hIndexDirMutex );
error7:
//    delete_semaphore( psVolume->av_hBlockListMutex );
error6:
//    delete_semaphore( psVolume->av_hJournalListsLock );
error5:
//    delete_semaphore( psVolume->av_hJournalLock );
error4:
//    free( psSuperBlock );
error3:
//    afs_delete_hash_table( &psVolume->av_sLogHashTable );
error2:
    free(psVolume);
    psVolume = NULL;
error1:
    printf("afs_mount() Flush cache blocks for our device\n");
//    flush_device_cache( nDev, false );
    printf("afs_mount() Shut down cache for our device\n");
//    shutdown_device_cache( nDev ); 

    close(nDev);
    return(nError);
}


/*****************************************************************************
 * NAME:
 * DESC:
 * NOTE:
 * SEE ALSO:
 *****************************************************************************/

bool afs_validate_superblock( AfsSuperBlock_s* psSuperBlock, bool bQuiet )
{
    bool        bResult = true;
 
    if ( psSuperBlock->as_nMagic1 != SUPER_BLOCK_MAGIC1 ) {
        if ( bQuiet ) return( false );
        printf( "Invalid super block magic1 = %lx\n", psSuperBlock->as_nMagic1 );
        bResult = false;
    }
    if ( psSuperBlock->as_nMagic2 != SUPER_BLOCK_MAGIC2 ) {
        if ( bQuiet ) return( false );
        printf( "Invalid super block magic2 = %lx\n", psSuperBlock->as_nMagic2 );
        bResult = false;
    }
    if ( psSuperBlock->as_nMagic3 != SUPER_BLOCK_MAGIC3 ) {
        if ( bQuiet ) return( false );
        printf( "Invalid super block magic3 = %lx\n", psSuperBlock->as_nMagic3 );
        bResult = false;
    }
    if ( psSuperBlock->as_nByteOrder != BO_LITTLE_ENDIAN && psSuperBlock->as_nByteOrder != BO_BIG_ENDIAN ) {
        if ( bQuiet ) return( false );
        printf( "Bad value in super block byte-order field %ld\n", psSuperBlock->as_nByteOrder );
        bResult = false;
    }
    if ( 1 << psSuperBlock->as_nBlockShift != psSuperBlock->as_nBlockSize ) {
        if ( bQuiet ) return( false );
        printf( "Inconsistent values in block-shift, and block-size values of superblock (sh=%ld/si=%ld)\n",
                psSuperBlock->as_nBlockShift, psSuperBlock->as_nBlockSize );
        bResult = false;
    }
    if ( psSuperBlock->as_nUsedBlocks > psSuperBlock->as_nNumBlocks ) {
        if ( bQuiet ) return( false );
        printf( "Super block has more free blocks than actual block (%Ld - %Ld)\n",
                psSuperBlock->as_nUsedBlocks, psSuperBlock->as_nNumBlocks );
        bResult = false; 
    }
    if ( psSuperBlock->as_nInodeSize != psSuperBlock->as_nBlockSize ) {
        if ( bQuiet ) return( false );
        printf( "Inconsistency between block-size (%ld) and inode-size (%ld)\n",
                psSuperBlock->as_nBlockSize, psSuperBlock->as_nInodeSize );
        bResult = false;
    }
    if ( 0 == psSuperBlock->as_nBlockSize ||
         (1 << psSuperBlock->as_nAllocGrpShift) / psSuperBlock->as_nBlockSize != psSuperBlock->as_nBlockPerGroup ) {
        if ( bQuiet ) return( false );
        printf( "Inconsistency between blocks per ag (%ld) and ag shift count (%ld)\n",
                psSuperBlock->as_nBlockPerGroup, psSuperBlock->as_nAllocGrpShift );
        bResult = false;
    }
    if ( psSuperBlock->as_nAllocGroupCount * psSuperBlock->as_nBlockPerGroup < psSuperBlock->as_nNumBlocks ) {
        if ( bQuiet ) return( false );
        printf( "Alloc group count (%ld) does not cower entire disk (%Ld)\n",
                psSuperBlock->as_nAllocGroupCount, psSuperBlock->as_nNumBlocks );
        bResult = false;
    }
    if ( psSuperBlock->as_sLogBlock.len != psSuperBlock->as_nLogSize ) {
        if ( bQuiet ) return( false );
        printf( "Mismatch between length in journal run %d an the length recorded in the superblock %d\n",
                psSuperBlock->as_sLogBlock.len, psSuperBlock->as_nLogSize );
        bResult = false;
    }
    if ( psSuperBlock->as_nValidLogBlocks > psSuperBlock->as_nLogSize ) {
        if ( bQuiet ) return( false );
        printf( "Superblock has more valid journal blocks (%d) than the total journal size (%d)\n",
                psSuperBlock->as_nValidLogBlocks, psSuperBlock->as_nLogSize );
        bResult = false;
    }
    return( bResult );
}


/*****************************************************************************
 * NAME:
 * DESC:
 * NOTE:
 * SEE ALSO:
 *****************************************************************************/
int afs_read_inode(int nDev, AfsSuperBlock_s *psSuperBlock, BlockRun_s *psBlockRun, AfsInode_s *psInode)
{
	off_t i;

	/* Figure which physical location to read */
	i = afs_run_to_num(psSuperBlock, psBlockRun) * psSuperBlock->as_nBlockSize;

	printf("Reading inode at: %lld\n", i);

	lseek(nDev, i, SEEK_SET);

	if(read(nDev, psInode, AFS_SUPERBLOCK_SIZE) != AFS_SUPERBLOCK_SIZE) {
		printf("Failed to read super block\n");
	}
}
