/*
 *  debugafs - file system debugger for AtheOS
 * 
 *  Copyright (C) 2001 Mike Bennett (mike@stressbunny.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
*/

#ifndef __DEBUGAFS_H__
#define __DEBUGAFS_H__

#define _TIME_H			// Mike: Kludge - get rid of once compiling on AtheOS

#include "../config.h"
#include "afs.h"
#include "stat.h"

/* May need to put this AFS defines in afs.h? */
#define DAFS_SUPERBLOCK_POS		1024

#define MAXCMDTOKENS			5	/* Max number of item in a command */
#define	MAXCMDLENGTH			512	/* Max length of a single line in interactive mode */

#define DFS_DEVICE_CLOSED		-1
#define DFS_NOREAD_SBLOCK		-2

enum {
	HELP_UNKNOWN,
	HELP_STANDARD,
	HELP_OPEN,
	HELP_SHOW_INODE_STATS
};

typedef struct
{
	char			*de_pzDevice;
	int				de_nDev;
	AfsVolume_s		*de_psVolume;
	AfsSuperBlock_s	*de_psSuperBlock;
} DebugAFS_s;

typedef struct  {
	long int		bi_nFlag;
	char			*bi_pzName;
} BinFlags_s;

/* For the mode in Inode */
BinFlags_s asInodeModes[] = {
	{S_IFLNK, "ISLNK"},
	{S_IFREG, "ISREG"},
	{S_IFDIR, "ISDIR"},
	{S_IFCHR, "ISCHR"},
	{S_IFBLK, "ISBLK"},
	{S_IFIFO, "ISFIFO"},
	{S_IFSOCK, "ISSOCK"},
	{(long int)NULL, NULL}
};

/* For the flags in Inode */
BinFlags_s asInodeFlags[] = {
	{INF_USED, "INF_USED"},
	{INF_ATTRIBUTES, "INF_ATTRIBUTES"},
	{INF_LOGGED, "INF_LOGGED"},
	{INF_NO_CACHE, "INF_NO_CACHE"},
	{INF_WAS_WRITTEN, "INF_WAS_WRITTEN"},
	{INF_NO_TRANSACTION, "INF_NO_TRANSACTION"},
	{INF_NOT_IN_DELME, "INF_NOT_IN_DELME"},
	{INF_STAT_CHANGED, "INF_STAT_CHANGED"},
	{(long int)NULL, NULL}
};

/* For the Index type in Inode */
BinFlags_s asInodeIType[] = {
	{e_KeyTypeInt32, "INT32"},
	{e_KeyTypeInt64, "INT64"},
	{e_KeyTypeFloat, "FLOAT"},
	{e_KeyTypeDouble, "DOUBLE"},
	{e_KeyTypeString, "STRING"},
	{(long int)NULL, "UNKNOWN"}
};


static inline int num_tokens(char **cmdtokens)
{
	int i;

	for(i = 0; cmdtokens[i] != NULL; i++);

	return(i);
}


int afs_read_superblock(int, AfsVolume_s *, AfsSuperBlock_s *);
bool afs_validate_superblock(AfsSuperBlock_s *, bool);

#endif __DEBUGAFS_H__